PROCEDURE WAIT_LOCK (NODE, LIMIT: INTEGER);
(*********************************************************************)
(*                                                                   *)
(*   THIS MODULE IS A COMPONENT OF DOOR SKELETON V1.0                *)
(*   COPYRIGHT (C) 1992 BERNIE GALLAGHER                             *)
(*                                                                   *)
(*********************************************************************)
(*                                                                   *)
(*   WAIT FOR THE LOCK FILE TO BECOME FREE                           *)
(*                                                                   *)
(*   LIMIT PARAMETER SPECIFIES THE NUMBER OF SECONDS YOU ARE         *)
(*   WILLING TO WAIT - IF THIS LIMIT IS EXCEEDED, A SYSTEM           *)
(*   DEADLOCK IS REPORTED AND THE PROGRAM HALTS AND CATCHES FIRE!    *)
(*                                                                   *)
(*   THE LOGIC HERE IS THAT EACH NODE HAS IT'S OWN LOCK FILE AND     *)
(*   CAN WRITE ONLY TO IT'S OWN LOCK FILE.  THEREFORE, THE           *)
(*   SEMAPHORES CANNOT POSSIBLY BE CORRUPTED BY TWO NODES RAISING    *)
(*   THEIR'S CONCURRENTLY.  THIS DESIGN IMPOSES A PERFORMANCE        *)
(*   PENALTY (MULTIPLE OPENS AND CLOSES WHILE CHECKING ALL THE       *)
(*   SEMAPHORES), BUT IT DOES NOT DEPEND ON ANY DOS BASED FILE       *)
(*   LOCKING INSTALLED.                                              *)
(*                                                                   *)
(*   A NODE ENTERING ITS CRITICAL SECTION FIRST RAISES ITS           *)
(*   OWN SEMAPHORE, THEN CHECKS ALL OTHER NODE'S SEMAPHORES.         *)
(*   IF IT FINDS THAT NO OTHER NODES ARE BUSY, IT KNOWS              *)
(*   THAT ALL OTHER NODES ARE BLOCKED UNTIL IT LOWERS IT'S           *)
(*   OWN AGAIN, AND IT IS SAFE FOR ITSELF TO PROCEED.                *)
(*   IF IT FINDS THAT ANOTHER NODE IS BUSY, IT LOWERS IT'S OWN,      *)
(*   DELAYS A CERTAIN AMOUNT OF TIME, THEN REPEATS THE CHECK.        *)
(*   IT WILL REPEAT THIS UNTIL EITHER ALL OTHER SEMAPHORES           *)
(*   ARE CLEAR, OR IT TIMES OUT.                                     *)
(*                                                                   *)
(*   SINCE IT WILL NEVER PROCEED UNLESS IT'S OWN SEMAPHORE IS UP     *)
(*   AND ALL OTHERS ARE DOWN, IT KNOWS THAT ONCE IT HAS RAISED       *)
(*   ITS OWN, IT IS SAFE TO PROCEED IF NO ONE ELSE IS BUSY.          *)
(*                                                                   *)
(*   EVEN IN A WORSE CASE SITUATION IN WHICH TWO NODES RAISE THEIR   *)
(*   SEMAPHORES CONCURRENTLY, BEFORE EITHER HAS CHECKED ANY OTHER    *)
(*   NODE'S SEMAPHORE, IT WILL RESULT IN A RACE CONDITION IN WHICH   *)
(*   ONE NODE WILL TIMEOUT AND ABORT ALLOWING THE OTHER TO PROCEED.  *)
(*   THE CHANCE OF A RACE CONDITION IS REDUCED BY A PRIORITIZING     *)
(*   SCHEME IN WHICH THE HIGHER THE NODE NUMBER, THE LONGER THE      *)
(*   DELAY BETWEEN LOCK ATTEMPTS - THUS GIVING LOWER NODE NUMBERS    *)
(*   HIGHER PRIORITY.                                                *)
(*                                                                   *)
(*********************************************************************)
VAR SAFE:       BOOLEAN;
VAR TICK:       REAL;
BEGIN

    SAFE := FALSE;
    TICK := TIME;
    WHILE (ABS (TIME - TICK) < LIMIT) AND (NOT (SAFE)) DO BEGIN

        SAFE := SET_LOCK (NODE);
        IF NOT (SAFE) THEN BEGIN

            FREE_LOCK (NODE);
            DELAY (TRUNC (200.0 * NODE));
            END;

        IF TICK > TIME THEN TICK := TIME;
        END;

    LOCKED := SAFE;
    IF NOT (SAFE) THEN BEGIN

        ALERT ('FILE SYSTEM DEADLOCK, PLEASE NOTIFY SYSOP!');
        DELAY (2000);
        STOP;
        END;

    END;

